@@ -3,7 +3,7 @@ use strict;
 #
 # auth_mysql.pl - authenticate user with DBD::MySQL
 #
-# 2002/03/07 Alan Sung - AlanSung@dragon2.net
+# 2002/03/07 Alan Sung, AlanSung.AT.dragon2.net
 #
 
 my $SQLHost = "dragon2.net";	# INSERT THE MYSQL SERVER IP HERE.
@@ -120,6 +120,8 @@ sub check_userpassword {
 	    } else {
                return (-4, 'Password incorrect');
 	    }
+         } else {
+            return(-3, "Unknown password type: $pass_type");
          }
       } else {
          return(-3, "Can't fetch SQL result: ".$sth->errstr());
@@ -138,7 +140,7 @@ sub change_userpassword {
    return (-2, "User or password is null") if (!$user||!$oldpassword||!$newpassword);
    return (-2, "Password too short") if (length($newpassword)<${$r_config}{'passwd_minlen'});
 
-   my ($ret, $errmsg)=check_userpassword($user, $oldpassword);
+   my ($ret, $errmsg)=check_userpassword($r_config, $user, $oldpassword);
    return($ret, $errmsg) if ($ret!=0);
 
    if ($pass_type eq "crypt") { # encrypt the passwd
