@@ -3,8 +3,8 @@ use strict;
 #
 # auth_pg.pl - authenticate user with DBD::Pg
 #
-# 2002/05/08 Oliver Smith - oliver@kfs.org
-# 2002/03/07 Alan Sung - AlanSung@dragon2.net (orig: auth_mysql.pl)
+# 2002/05/08 Oliver Smith, oliver.AT.kfs.org
+# 2002/03/07 Alan Sung, AlanSung.AT.dragon2.net (orig: auth_mysql.pl)
 #
 
 my $SQLHost = "localhost";	# INSERT THE PGSQL SERVER IP HERE.
@@ -123,6 +123,8 @@ sub check_userpassword {
 	    } else {
                return (-4, 'Password incorrect');
 	    }
+         } else {
+            return (-3, "Unknown password type: $pass_type");
          }
       }
    }
@@ -139,7 +141,7 @@ sub change_userpassword {
    return (-2, "User or password is null") if (!$user||!$oldpassword||!$newpassword);
    return (-2, "Password too short") if (length($newpassword)<${$r_config}{'passwd_minlen'});
 
-   my ($ret, $errmsg)=check_userpassword($user, $oldpassword);
+   my ($ret, $errmsg)=check_userpassword($r_config, $user, $oldpassword);
    return($ret, $errmsg) if ($ret!=0);
 
    if ($pass_type eq "crypt") { # encrypt the passwd
