@@ -6,10 +6,10 @@ use strict;
 #
 # Version 1.15 Aug 6, 2002
 #
-# 2002/08/06 Trevor.Paquette@TeraGo.ca
+# 2002/08/06 Trevor.Paquette.AT.TeraGo.ca
 #            Fix comments
-# 2002/07/16 Trevor.Paquette@TeraGo.ca (add check for cobalt security)
-# 2001/12/20 tung@turtle.ee.ncku.edu.tw (orig: auth_unix.pl)
+# 2002/07/16 Trevor.Paquette.AT.TeraGo.ca (add check for cobalt security)
+# 2001/12/20 tung.AT.turtle.ee.ncku.edu.tw (orig: auth_unix.pl)
 #
 
 #
@@ -37,7 +37,7 @@ use strict;
 #
 # $unix_passwdfile_plaintext : the plaintext file containing all usernames
 #                              and related uid, gid, homedir, shell info.
-#                              The deault is /etc/passwd on most unix systems.
+#                              The default is /etc/passwd on most unix systems.
 # $unix_passwdfile_encrypted : the file containing all usernames and
 #                              their corresponding encrypted passwords.
 # $unix_passwdmkdb : The command executed after any password modification
@@ -67,7 +67,7 @@ sub get_userinfo {
       ($uid, $gid, $realname, $homedir)= (getpwnam($user))[2,3,6,7];
    } else {
       if ($unix_passwdfile_plaintext=~/\|/) { # maybe NIS, try getpwnam first
-         ($uid, $gid, $realname, $homedir)= (getpwnam($user))[2,3,6,7]; 
+         ($uid, $gid, $realname, $homedir)= (getpwnam($user))[2,3,6,7];
       }
       if ($uid eq "") { # else, open file directly
          ($uid, $gid, $realname, $homedir)= (getpwnam_file($user, $unix_passwdfile_plaintext))[2,3,6,7];
@@ -123,7 +123,7 @@ sub check_userpassword {
 
    # a file should be locked only if it is local accessable
    if (-f $unix_passwdfile_encrypted) {
-      filelock("$unix_passwdfile_encrypted", LOCK_SH) or 
+      filelock("$unix_passwdfile_encrypted", LOCK_SH) or
          return (-3, "Couldn't get read lock on $unix_passwdfile_encrypted");
    }
    if ( ! open (PASSWD, "$unix_passwdfile_encrypted") ) {
@@ -255,13 +255,13 @@ sub change_userpassword {
       return (-3, "Unable to match entry for modification");
    }
 
-   open(TMP, ">$unix_passwdfile_encrypted.tmp.$$") || goto authsys_error;
-   print TMP $content || goto authsys_error;
-   close(TMP) || goto authsys_error;
+   open(TMP, ">$unix_passwdfile_encrypted.tmp.$$") or goto authsys_error;
+   print TMP $content or goto authsys_error;
+   close(TMP) or goto authsys_error;
 
    if ($unix_passwdmkdb ne "" && $unix_passwdmkdb ne "none" ) {
       # disable outside $SIG{CHLD} handler temporarily for system() return value
-      local $SIG{CHLD}; undef $SIG{CHLD}; 
+      local $SIG{CHLD}; undef $SIG{CHLD};
       # update passwd and db with pwdmkdb program
       if ( system("$unix_passwdmkdb $unix_passwdfile_encrypted.tmp.$$")!=0 ) {
          goto authsys_error;
@@ -271,7 +271,7 @@ sub change_userpassword {
       my ($fmode, $fuid, $fgid) = (stat($unix_passwdfile_encrypted))[2,4,5];
       chown($fuid, $fgid, "$unix_passwdfile_encrypted.tmp.$$");
       chmod($fmode, "$unix_passwdfile_encrypted.tmp.$$");
-      rename("$unix_passwdfile_encrypted.tmp.$$", $unix_passwdfile_encrypted) || goto authsys_error;
+      rename("$unix_passwdfile_encrypted.tmp.$$", $unix_passwdfile_encrypted) or goto authsys_error;
    }
    filelock("$unix_passwdfile_encrypted", LOCK_UN);
    return (0, "");
