@@ -1,7 +1,7 @@
 #
 # filelock.pl - filelock routines for local or nfs server
 #
-# 2001/04/25 tung@turtle.ee.ncku.edu.tw
+# 2001/04/25 tung.AT.turtle.ee.ncku.edu.tw
 #
 package openwebmail::filelock;
 
@@ -24,11 +24,12 @@ sub closeall {
 # it opens the file to get the handle if need,
 # than do lock operation on the related filehandle
 sub flock_lock {
-   my ($filename, $lockflag)=@_;
+   my ($filename, $lockflag, $perm)=@_;
 
-   ($filename =~ /^(.+)$/) && ($filename = $1);	# untaint...
+   $filename=untaint($filename);
    if ( (! -e $filename) && $lockflag ne LOCK_UN) {
-      sysopen(F, $filename, O_RDWR|O_CREAT, 0600) or return 0; # create file for lock
+      $perm=0600 if (!$perm);
+      sysopen(F, $filename, O_RDWR|O_CREAT, $perm) or return 0; # create file for lock
       close(F);
    }
 
@@ -81,12 +82,13 @@ sub flock_lock {
 # filename.lock and filename.lock.lock could not be created, this routine will
 # grant all shared lock and deny all exclusive lock.
 sub dotfile_lock {
-   my ($filename, $lockflag)=@_;
+   my ($filename, $lockflag, $perm)=@_;
    return 1 unless ($lockflag & (LOCK_SH|LOCK_EX|LOCK_UN));
 
-   ($filename =~ /^(.+)$/) && ($filename = $1);	# untaint...
+   $filename=untaint($filename);
    if ( (! -e $filename) && $lockflag ne LOCK_UN) {
-      sysopen(F, $filename, O_RDWR|O_CREAT, 0600) or return 0; # create file for lock
+      $perm=0600 if (!$perm);
+      sysopen(F, $filename, O_RDWR|O_CREAT, $perm) or return 0; # create file for lock
       close(F);
    }
 
@@ -96,7 +98,7 @@ sub dotfile_lock {
       $ldepth++; return (0) if ($ldepth>8);		# link to deep
       $filename=readlink($filename);
    }
-   ($filename =~ /^(.+)$/) && ($filename = $1);		# untaint ...
+   $filename=untaint($filename);
 
    my $oldumask=umask(0111);
    my ($endtime, $mode, $count);
@@ -202,7 +204,7 @@ sub dotfile_lock {
 # _lock and _unlock are used to lock/unlock xxx.lock
 sub _lock {
    my ($filename, $staletimeout)=@_;
-   ($filename =~ /^(.+)$/) && ($filename = $1);		# untaint ...
+   $filename=untaint($filename);
 
    $staletimeout=60 if $staletimeout eq 0;
    if ( my $t=(stat("$filename.lock"))[9] ) {
@@ -221,9 +223,7 @@ sub _lock {
 }
 
 sub _unlock {
-   my ($filename)=$_[0];
-   ($filename =~ /^(.+)$/) && ($filename = $1);		# untaint ...
-
+   my $filename=untaint($_[0]);
    if ( unlink("$filename.lock") ) {
       return 1;
    } else {
@@ -235,4 +235,10 @@ sub _unlock {
    }
 }
 
+sub untaint {
+   local $_ = shift;
+   m/^(.*)$/;
+   return $1;
+}
+
 1;
