@@ -1,8 +1,8 @@
 #
 # mailfilter.pl - mail filter routines
 #
-# 2001/11/13 Ebola@turtle.ee.ncku.edu.tw
-#            tung@turtle.ee.ncku.edu.tw
+# 2001/11/13 Ebola.AT.turtle.ee.ncku.edu.tw
+#            tung.AT.turtle.ee.ncku.edu.tw
 #
 
 use strict;
@@ -16,8 +16,9 @@ use vars qw(%config);
 # there are 4 op for a msg: 'copy', 'move', 'delete' and 'keep'
 sub mailfilter {
    my ($user, $folder, $folderdir, $r_validfolders, $prefs_regexmatch,
-	$filter_repeatlimit, $filter_fakedsmtp,
-	$filter_fakedfrom, $filter_fakedexecontenttype)=@_;
+	$filter_repeatlimit, $filter_badformatfrom,
+	$filter_fakedsmtp, $filter_fakedfrom, $filter_fakedexecontenttype)=@_;
+
    my ($folderfile, $headerdb)=get_folderfile_headerdb($user, $folder);
    my @filterrules;
    my $folderhandle=do { local *FH };
@@ -30,6 +31,7 @@ sub mailfilter {
    if ( ! -f $folderfile ) {
       return 0;
    }
+
    ## check .filter_check ##
    if ( -f "$folderdir/.filter.check" ) {
       my $checkinfo;
@@ -44,7 +46,7 @@ sub mailfilter {
    }
 
    ## get @filterrules ##
-   if ( -f "$folderdir/.filter.book" ) {
+   if ( $config{'enable_userfilter'} && -f "$folderdir/.filter.book" ) {
       open (FILTER,"$folderdir/.filter.book") or return -2;
       while (<FILTER>) {
          chomp($_);
@@ -64,17 +66,14 @@ sub mailfilter {
       }
    }
 
-   if ( ! -e "$folderdir/.filter.book$config{'dbm_ext'}" ) {
-      dbmopen (%FTDB, "$folderdir/.filter.book$config{'dbmopen_ext'}", 0600);
-      dbmclose(%FTDB);
-   }
-   if (!$config{'dbmopen_haslock'}) {
-      filelock("$folderdir/.filter.book$config{'dbm_ext'}", LOCK_EX) or return -3;
-   }
-   dbmopen (%FTDB, "$folderdir/.filter.book$config{'dbmopen_ext'}", 0600);
+   open_dbm(\%FTDB, "$folderdir/.filter.book", LOCK_EX) or return -3;
 
-   filelock($folderfile, LOCK_EX|LOCK_NB) or return -4;
+   if (! filelock($folderfile, LOCK_EX|LOCK_NB)) {
+      close_dbm(\%FTDB, "$folderdir/.filter.book");
+      return -4;
+   }
    if (update_headerdb($headerdb, $folderfile)<0) {
+      close_dbm(\%FTDB, "$folderdir/.filter.book");
       filelock($folderfile, LOCK_UN);
       writelog("db error - Couldn't update index db $headerdb$config{'dbm_ext'}");
       writehistory("db error - Couldn't update index db $headerdb$config{'dbm_ext'}");
@@ -82,10 +81,11 @@ sub mailfilter {
    }
    open ($folderhandle, "+<$folderfile") or return -5;
    @allmessageids=get_messageids_sorted_by_offset($headerdb);
-   if (!$config{'dbmopen_haslock'}) {
-      filelock("$headerdb$config{'dbm_ext'}", LOCK_EX) or return -6;
+   if (!open_dbm(\%HDB, $headerdb, LOCK_EX)) {
+      close_dbm(\%FTDB, "$folderdir/.filter.book");
+      filelock($folderfile, LOCK_UN);
+      return -6;
    }
-   dbmopen (%HDB, "$headerdb$config{'dbmopen_ext'}", 0600);
 
    my ($blockstart, $blockend, $writepointer)=(0,0,0);
    my %filtered=();
@@ -104,8 +104,7 @@ sub mailfilter {
       if ($filter_repeatlimit>0) {
          # store msgid with same '$from:$subject' to same array
          if (! defined($repeatlists{"$attr[$_FROM]:$attr[$_SUBJECT]"}) ) {
-            my @a=();
-            $repeatlists{"$attr[$_FROM]:$attr[$_SUBJECT]"}=\@a;
+            $repeatlists{"$attr[$_FROM]:$attr[$_SUBJECT]"}=[];	# reference of null array
          }
          push (@{$repeatlists{"$attr[$_FROM]:$attr[$_SUBJECT]"}}, $allmessageids[$i] );
       }
@@ -145,10 +144,10 @@ sub mailfilter {
 
             if ( $rules eq 'from' || $rules eq 'to' || $rules eq 'subject' ) {
                my %index=(from=>$_FROM, to=>$_TO, subject=>$_SUBJECT);
-               if (   ($include eq 'include' && $regexmatch && $attr[$index{$rules}] =~ /$text/i)
-                   || ($include eq 'include' && $attr[$index{$rules}] =~ /\Q$text\E/i)
-                   || ($include eq 'exclude' && $regexmatch && $attr[$index{$rules}] !~ /$text/i)
-                   || ($include eq 'exclude' && $attr[$index{$rules}] !~ /\Q$text\E/i)  ) {
+               if (   ($include eq 'include' && $regexmatch  && $attr[$index{$rules}] =~ /$text/i)
+                   || ($include eq 'include' && !$regexmatch && $attr[$index{$rules}] =~ /\Q$text\E/i)
+                   || ($include eq 'exclude' && $regexmatch  && $attr[$index{$rules}] !~ /$text/i)
+                   || ($include eq 'exclude' && !$regexmatch && $attr[$index{$rules}] !~ /\Q$text\E/i)  ) {
                   my ($matchcount, $matchdate)=split(":", $FTDB{"$rules\@\@\@$include\@\@\@$text\@\@\@$destination"});
                   $matchcount++; $matchdate=gmtime2dateserial();
                   $FTDB{"$rules\@\@\@$include\@\@\@$text\@\@\@$destination"}="$matchcount:$matchdate";
@@ -193,10 +192,10 @@ sub mailfilter {
                }
 
                $header=~s/\n / /g;	# handle folding roughly
-               if (  ( $include eq 'include' && $regexmatch && $header =~ /$text/im )
-                   ||( $include eq 'include' && $header =~ /\Q$text\E/im )
-                   ||( $include eq 'exclude' && $regexmatch && $header !~ /$text/im )
-                   ||( $include eq 'exclude' && $header !~ /\Q$text\E/im ) ) {
+               if (  ( $include eq 'include' && $regexmatch  && $header =~ /$text/im )
+                   ||( $include eq 'include' && !$regexmatch && $header =~ /\Q$text\E/im )
+                   ||( $include eq 'exclude' && $regexmatch  && $header !~ /$text/im )
+                   ||( $include eq 'exclude' && !$regexmatch && $header !~ /\Q$text\E/im ) ) {
                   my ($matchcount, $matchdate)=split(":", $FTDB{"$rules\@\@\@$include\@\@\@$text\@\@\@$destination"});
                   $matchcount++; $matchdate=gmtime2dateserial();
                   $FTDB{"$rules\@\@\@$include\@\@\@$text\@\@\@$destination"}="$matchcount:$matchdate";
@@ -238,10 +237,10 @@ sub mailfilter {
                foreach my $relay (@{$r_smtprelays}) {
                   $smtprelays.="$relay, ${$r_connectfrom}{$relay}, ${$r_byas}{$relay}, ";
                }
-               if (  ( $include eq 'include' && $regexmatch && $smtprelays =~ /$text/im )
-                   ||( $include eq 'include' && $smtprelays =~ /\Q$text\E/im )
-                   ||( $include eq 'exclude' && $regexmatch && $smtprelays !~ /$text/im )
-                   ||( $include eq 'exclude' && $smtprelays !~ /\Q$text\E/im ) ) {
+               if (  ( $include eq 'include' && $regexmatch  && $smtprelays =~ /$text/im )
+                   ||( $include eq 'include' && !$regexmatch && $smtprelays =~ /\Q$text\E/im )
+                   ||( $include eq 'exclude' && $regexmatch  && $smtprelays !~ /$text/im )
+                   ||( $include eq 'exclude' && !$regexmatch && $smtprelays !~ /\Q$text\E/im ) ) {
                   my ($matchcount, $matchdate)=split(":", $FTDB{"$rules\@\@\@$include\@\@\@$text\@\@\@$destination"});
                   $matchcount++; $matchdate=gmtime2dateserial();
                   $FTDB{"$rules\@\@\@$include\@\@\@$text\@\@\@$destination"}="$matchcount:$matchdate";
@@ -291,10 +290,10 @@ sub mailfilter {
                   $is_body_decoded=1;
                }
 
-               if (  ( $include eq 'include' && $regexmatch && $body =~ /$text/im )
-                   ||( $include eq 'exclude' && $body !~ /\Q$text\E/im )
-                   ||( $include eq 'include' && $regexmatch && $body =~ /$text/im )
-                   ||( $include eq 'exclude' && $body !~ /\Q$text\E/im ) ) {
+               if (  ( $include eq 'include' && $regexmatch  && $body =~ /$text/im )
+                   ||( $include eq 'exclude' && !$regexmatch && $body !~ /\Q$text\E/im )
+                   ||( $include eq 'include' && $regexmatch  && $body =~ /$text/im )
+                   ||( $include eq 'exclude' && !$regexmatch && $body !~ /\Q$text\E/im ) ) {
                   my ($matchcount, $matchdate)=split(":", $FTDB{"$rules\@\@\@$include\@\@\@$text\@\@\@$destination"});
                   $matchcount++; $matchdate=gmtime2dateserial();
                   $FTDB{"$rules\@\@\@$include\@\@\@$text\@\@\@$destination"}="$matchcount:$matchdate";
@@ -340,10 +339,10 @@ sub mailfilter {
                foreach my $r_attachment (@{$r_attachments}) {
                   if ( ${$r_attachment}{contenttype} =~ /^text/i ||
                        ${$r_attachment}{contenttype} eq "N/A" ) { # read all for text/plain. text/html
-                     if (  ( $include eq 'include' && $regexmatch && ${${$r_attachment}{r_content}} =~ /$text/im )
-                         ||( $include eq 'include' && ${${$r_attachment}{r_content}} =~ /\Q$text\E/im )
-                         ||( $include eq 'exclude' && $regexmatch && ${${$r_attachment}{r_content}} !~ /$text/im )
-                         ||( $include eq 'exclude' && ${${$r_attachment}{r_content}} !~ /\Q$text\E/im )  ) {
+                     if (  ( $include eq 'include' && $regexmatch  && ${${$r_attachment}{r_content}} =~ /$text/im )
+                         ||( $include eq 'include' && !$regexmatch && ${${$r_attachment}{r_content}} =~ /\Q$text\E/im )
+                         ||( $include eq 'exclude' && $regexmatch  && ${${$r_attachment}{r_content}} !~ /$text/im )
+                         ||( $include eq 'exclude' && !$regexmatch && ${${$r_attachment}{r_content}} !~ /\Q$text\E/im )  ) {
                         my ($matchcount, $matchdate)=split(":", $FTDB{"$rules\@\@\@$include\@\@\@$text\@\@\@$destination"});
                         $matchcount++; $matchdate=gmtime2dateserial();
                         $FTDB{"$rules\@\@\@$include\@\@\@$text\@\@\@$destination"}="$matchcount:$matchdate";
@@ -385,10 +384,10 @@ sub mailfilter {
                }
                # check attachments
                foreach my $r_attachment (@{$r_attachments}) {
-                  if (   ( $include eq 'include' && $regexmatch && ${$r_attachment}{filename} =~ /$text/i )
-                       ||( $include eq 'include' && ${$r_attachment}{filename} =~ /\Q$text\E/i )
-                       ||( $include eq 'exclude' && $regexmatch && ${$r_attachment}{filename} !~ /$text/i )
-                       ||( $include eq 'exclude' && ${$r_attachment}{filename} !~ /\Q$text\E/i )  ) {
+                  if (   ( $include eq 'include' && $regexmatch  && ${$r_attachment}{filename} =~ /$text/i )
+                       ||( $include eq 'include' && !$regexmatch && ${$r_attachment}{filename} =~ /\Q$text\E/i )
+                       ||( $include eq 'exclude' && $regexmatch  && ${$r_attachment}{filename} !~ /$text/i )
+                       ||( $include eq 'exclude' && !$regexmatch && ${$r_attachment}{filename} !~ /\Q$text\E/i )  ) {
                      my ($matchcount, $matchdate)=split(":", $FTDB{"$rules\@\@\@$include\@\@\@$text\@\@\@$destination"});
                      $matchcount++; $matchdate=gmtime2dateserial();
                      $FTDB{"$rules\@\@\@$include\@\@\@$text\@\@\@$destination"}="$matchcount:$matchdate";
@@ -421,85 +420,37 @@ sub mailfilter {
 
          } # end @filterrules
 
-         # filter message with faked exe contenttype if msg is not moved or deleted
-         if ( $filter_fakedexecontenttype &&
-              !($matched && ($op eq 'move' || $op eq 'delete')) &&
-              !$reserved_in_inbox ) {
-            if ($currmessage eq "") {
-               seek($folderhandle, $attr[$_OFFSET], 0);
-               read($folderhandle, $currmessage, $attr[$_SIZE]);
+         if ($config{'enable_smartfilter'}) {
+
+            # bypass smart filters for good messages
+            if ($config{'smartfilter_bypass_goodmessage'} &&
+                 !($matched && ($op eq 'move' || $op eq 'delete')) &&
+                 !$reserved_in_inbox ) {
+               if ($currmessage eq "") {
+                  seek($folderhandle, $attr[$_OFFSET], 0);
+                  read($folderhandle, $currmessage, $attr[$_SIZE]);
+               }
+               if ($is_message_parsed==0) {
+                  ($header, $body, $r_attachments)=parse_rfc822block(\$currmessage);
+                  $is_message_parsed=1;
+               }
+               if ( ($header=~/^X\-Mailer: Open WebMail/m && $header=~/^X\-OriginatingIP: /m) ||
+                    ($header=~/^In\-Reply\-To: /m && $header=~/^References: /m) ) {
+                  $reserved_in_inbox=1;
+               }
             }
-            if ($is_message_parsed==0) {
-               ($header, $body, $r_attachments)=parse_rfc822block(\$currmessage);
-               $is_message_parsed=1;
-            }
-            # check executable attachment and contenttype
-            foreach my $r_attachment (@{$r_attachments}) {
-               if ( ${$r_attachment}{filename} =~ /\.(?:exe|com|bat|pif|lnk|scr)$/i &&
-                    ${$r_attachment}{contenttype} !~ /application\/octet\-stream/i &&
-                    ${$r_attachment}{contenttype} !~ /application\/x\-msdownload/i ) {
-                  my ($matchcount, $matchdate)=split(":", $FTDB{"filter_fakedexecontenttype"});
+
+            # filter message with bad format from if msg is not moved or deleted
+            if ( $filter_badformatfrom &&
+                 !($matched && ($op eq 'move' || $op eq 'delete')) &&
+                 !$reserved_in_inbox ) {
+               my $from=(email2nameaddr($attr[$_FROM]))[1]; $from=~s/\@.*$//;
+               if ($from=~/[^\d\w\-\._]/ ||
+                   $from=~/^\d/ ||
+                   ($from=~/\d/ && $from=~/\./) ) {
+                  my ($matchcount, $matchdate)=split(":", $FTDB{'filter_badformatfrom'});
                   $matchcount++; $matchdate=gmtime2dateserial();
-                  $FTDB{"filter_fakedexecontenttype"}="$matchcount:$matchdate";
-
-                  $matched = 1;
-                  last;	# leave attachments check from one message
-               }
-            }
-            if ($matched) {
-               my $append=append_message_to_folder($allmessageids[$i],
-   					\@attr, \$currmessage, 'mail-trash',
-   					$r_validfolders, $user);
-               if ($append>=0) {
-                  $op='move';
-                  $matched=1;
-               } else {
-                  $matched=0;	# match not counted if move failed
-               }
-            }
-         } # end of checking faked exe contenttype
-
-         # filter message whose from: is different than the envelope email address
-         if ( $filter_fakedfrom &&
-              !($matched && ($op eq 'move' || $op eq 'delete')) &&
-              !$reserved_in_inbox ) {
-            if ($currmessage eq "") {
-               seek($folderhandle, $attr[$_OFFSET], 0);
-               read($folderhandle, $currmessage, $attr[$_SIZE]);
-            }
-            if ($is_message_parsed==0) {
-               ($header, $body, $r_attachments)=parse_rfc822block(\$currmessage);
-               $is_message_parsed=1;
-            }
-
-            my $is_tmda=0;	# skip faked from check if TMDA msg
-            if ($header=~/^\QX-Delivery-Agent: TMDA\E/m &&
-                $header=~/^\QPrecedence: bulk\E/m &&
-                $allmessageids[$i]=~/\Q.TMDA@\E/ ) {
-               $is_tmda=1;
-            }
-            if (! $is_tmda) {
-               my $envelopefrom="";
-               foreach (split(/\n/, $header)) {
-                  if (/\(envelope\-from ([^\s]+).*\)/) {
-                     $envelopefrom=$1; last;
-                  }
-               }
-               if ($envelopefrom eq "") {
-                  $envelopefrom=$1 if ($header=~/^From ([^\s]+)/);
-               }
-
-               # compare user and domain independently
-               my ($hdr_user, $hdr_domain)=split(/\@/, (email2nameaddr($attr[$_FROM]))[1]);
-               my ($env_user, $env_domain)=split(/\@/, $envelopefrom);
-               if ( $hdr_user ne $env_user ||
-                   ($hdr_domain ne "" &&
-                    $env_domain ne "" &&
-                    $hdr_domain!~/\Q$env_domain\E/i &&
-                    $env_domain!~/\Q$hdr_domain\E/i) ) {
-                  my ($matchcount, $matchdate)=split(":", $FTDB{"filter_fakedfrom"});
-                  $matchcount++; $matchdate=gmtime2dateserial();
-                  $FTDB{"filter_fakedfrom"}="$matchcount:$matchdate";
+                  $FTDB{"filter_badformatfrom"}="$matchcount:$matchdate";
 
                   my $append=append_message_to_folder($allmessageids[$i],
       					\@attr, \$currmessage, 'mail-trash',
@@ -507,77 +458,174 @@ sub mailfilter {
                   if ($append>=0) {
                      $op='move';
                      $matched=1;
+                  } else {
+                     $matched=0;	# match not counted if move failed
                   }
                }
-            }
-         } # end of checking faked from
+            } # end of checking bad format from
 
-         # filter message from smtprelay with faked name if msg is not moved or deleted
-         if ( $filter_fakedsmtp &&
-              !($matched && ($op eq 'move' || $op eq 'delete')) &&
-              !$reserved_in_inbox ) {
-            if ($currmessage eq "") {
-               seek($folderhandle, $attr[$_OFFSET], 0);
-               read($folderhandle, $currmessage, $attr[$_SIZE]);
-            }
-            if ($is_message_parsed==0) {
-               ($header, $body, $r_attachments)=parse_rfc822block(\$currmessage);
-               $is_message_parsed=1;
-            }
-            if (!defined($r_smtprelays) ) {
-               ($r_smtprelays, $r_connectfrom, $r_byas)=get_smtprelays_connectfrom_byas($header);
-            }
+            # filter message with faked exe contenttype if msg is not moved or deleted
+            if ( $filter_fakedexecontenttype &&
+                 !($matched && ($op eq 'move' || $op eq 'delete')) &&
+                 !$reserved_in_inbox ) {
+               if ($currmessage eq "") {
+                  seek($folderhandle, $attr[$_OFFSET], 0);
+                  read($folderhandle, $currmessage, $attr[$_SIZE]);
+               }
+               if ($is_message_parsed==0) {
+                  ($header, $body, $r_attachments)=parse_rfc822block(\$currmessage);
+                  $is_message_parsed=1;
+               }
+               # check executable attachment and contenttype
+               foreach my $r_attachment (@{$r_attachments}) {
+                  if ( ${$r_attachment}{filename} =~ /\.(?:exe|com|bat|pif|lnk|scr)$/i &&
+                       ${$r_attachment}{contenttype} !~ /application\/octet\-stream/i &&
+                       ${$r_attachment}{contenttype} !~ /application\/x\-msdownload/i ) {
+                     my ($matchcount, $matchdate)=split(":", $FTDB{'filter_fakedexecontenttype'});
+                     $matchcount++; $matchdate=gmtime2dateserial();
+                     $FTDB{"filter_fakedexecontenttype"}="$matchcount:$matchdate";
 
-            # move msg to trash if the first relay has invalid/faked hostname
-            if ( defined(${$r_smtprelays}[0]) ) {
-               my $relay=${$r_smtprelays}[0];
-               my $connectfrom=${$r_connectfrom}{$relay};
-               my $byas=${$r_byas}{$relay};
+                     $matched = 1;
+                     last;	# leave attachments check from one message
+                  }
+               }
+               if ($matched) {
+                  my $append=append_message_to_folder($allmessageids[$i],
+      					\@attr, \$currmessage, 'mail-trash',
+      					$r_validfolders, $user);
+                  if ($append>=0) {
+                     $op='move';
+                     $matched=1;
+                  } else {
+                     $matched=0;	# match not counted if move failed
+                  }
+               }
+            } # end of checking faked exe contenttype
 
-               my $is_private=0;
-               if ($connectfrom =~ /\[10\./ ||
-                   $connectfrom =~ /\[172\.[1-3][0-9]\./ ||
-                   $connectfrom =~ /\[192\.168\./ ||
-                   $connectfrom =~ /\[127\.0\./ ) {
-                   $is_private=1;
+            # filter message whose from: is different than the envelope email address
+            if ( $filter_fakedfrom &&
+                 !($matched && ($op eq 'move' || $op eq 'delete')) &&
+                 !$reserved_in_inbox ) {
+               if ($currmessage eq "") {
+                  seek($folderhandle, $attr[$_OFFSET], 0);
+                  read($folderhandle, $currmessage, $attr[$_SIZE]);
+               }
+               if ($is_message_parsed==0) {
+                  ($header, $body, $r_attachments)=parse_rfc822block(\$currmessage);
+                  $is_message_parsed=1;
                }
 
-               my $is_valid;
-               my @compare=( namecompare($connectfrom, $relay),
-                             namecompare($byas, $relay),
-                             namecompare($connectfrom, $byas) );
-               if ( $compare[0]>0 || $compare[1]>0 || $compare[2]>0 ||
-                   ($compare[0]==0 && $compare[1]==0 && $compare[2]==0) ) {
-                  $is_valid=1;
-               } else {	# all <=0 and at least one < 0
-                  $is_valid=0;
+               my $is_software_generated=0;	# skip faked from check for msg generated by some software
+               if ( ($header=~/^\QX-Delivery-Agent: TMDA\E/m &&
+                     $header=~/^\QPrecedence: bulk\E/m &&
+                     $allmessageids[$i]=~/\Q.TMDA@\E/) ||	# TMDA
+                    ($header=~/^\QManaged-by: RT\E/m &&
+                     $header=~/^\QRT-Ticket: \E/m &&
+                     $header=~/^\QPrecedence: bulk\E/m) ) {	# Request Tracker
+                  $is_software_generated=1;
+               }
+               if (! $is_software_generated) {
+                  my $envelopefrom="";
+                  foreach (split(/\n/, $header)) {
+                     if (/\(envelope\-from ([^\s]+).*\)/) {
+                        $envelopefrom=$1; last;
+                     }
+                  }
+                  if ($envelopefrom eq "") {
+                     $envelopefrom=$1 if ($header=~/^From ([^\s]+)/);
+                  }
+
+                  # compare user and domain independently
+                  my ($hdr_user, $hdr_domain)=split(/\@/, (email2nameaddr($attr[$_FROM]))[1]);
+                  my ($env_user, $env_domain)=split(/\@/, $envelopefrom);
+                  if ( $hdr_user ne $env_user ||
+                      ($hdr_domain ne "" &&
+                       $env_domain ne "" &&
+                       $hdr_domain!~/\Q$env_domain\E/i &&
+                       $env_domain!~/\Q$hdr_domain\E/i) ) {
+                     my ($matchcount, $matchdate)=split(":", $FTDB{'filter_fakedfrom'});
+                     $matchcount++; $matchdate=gmtime2dateserial();
+                     $FTDB{"filter_fakedfrom"}="$matchcount:$matchdate";
+
+                     my $append=append_message_to_folder($allmessageids[$i],
+         					\@attr, \$currmessage, 'mail-trash',
+         					$r_validfolders, $user);
+                     if ($append>=0) {
+                        $op='move';
+                        $matched=1;
+                     }
+                  }
+               }
+            } # end of checking faked from
+
+            # filter message from smtprelay with faked name if msg is not moved or deleted
+            if ( $filter_fakedsmtp &&
+                 !($matched && ($op eq 'move' || $op eq 'delete')) &&
+                 !$reserved_in_inbox ) {
+               if ($currmessage eq "") {
+                  seek($folderhandle, $attr[$_OFFSET], 0);
+                  read($folderhandle, $currmessage, $attr[$_SIZE]);
+               }
+               if ($is_message_parsed==0) {
+                  ($header, $body, $r_attachments)=parse_rfc822block(\$currmessage);
+                  $is_message_parsed=1;
+               }
+               if (!defined($r_smtprelays) ) {
+                  ($r_smtprelays, $r_connectfrom, $r_byas)=get_smtprelays_connectfrom_byas($header);
                }
 
-               # the last relay is the mail server
-               my $dstdomain=domain(${$r_smtprelays}[$#{$r_smtprelays}]);
+               # move msg to trash if the first relay has invalid/faked hostname
+               if ( defined(${$r_smtprelays}[0]) ) {
+                  my $relay=${$r_smtprelays}[0];
+                  my $connectfrom=${$r_connectfrom}{$relay};
+                  my $byas=${$r_byas}{$relay};
+
+                  my $is_private=0;
+                  if ($connectfrom =~ /\[10\./ ||
+                      $connectfrom =~ /\[172\.[1-3][0-9]\./ ||
+                      $connectfrom =~ /\[192\.168\./ ||
+                      $connectfrom =~ /\[127\.0\./ ) {
+                      $is_private=1;
+                  }
+
+                  my $is_valid;
+                  my @compare=( namecompare($connectfrom, $relay),
+                                namecompare($byas, $relay),
+                                namecompare($connectfrom, $byas) );
+                  if ( $compare[0]>0 || $compare[1]>0 || $compare[2]>0 ||
+                      ($compare[0]==0 && $compare[1]==0 && $compare[2]==0) ) {
+                     $is_valid=1;
+                  } else {	# all <=0 and at least one < 0
+                     $is_valid=0;
+                  }
+
+                  # the last relay is the mail server
+                  my $dstdomain=domain(${$r_smtprelays}[$#{$r_smtprelays}]);
 #log_time("relay $relay");
 #log_time("connectfrom $connectfrom");
 #log_time("byas $byas");
 #log_time("dstdomain $dstdomain");
 #log_time("is_private $is_private");
 #log_time("is_valid $is_valid\n");
-               if ($connectfrom !~ /\Q$dstdomain\E/i &&
-                   !$is_private &&
-                   !$is_valid ) {
-                  my ($matchcount, $matchdate)=split(":", $FTDB{"filter_fakedsmtp"});
-                  $matchcount++; $matchdate=gmtime2dateserial();
-                  $FTDB{"filter_fakedsmtp"}="$matchcount:$matchdate";
+                  if ($connectfrom !~ /\Q$dstdomain\E/i &&
+                      !$is_private &&
+                      !$is_valid ) {
+                     my ($matchcount, $matchdate)=split(":", $FTDB{'filter_fakedsmtp'});
+                     $matchcount++; $matchdate=gmtime2dateserial();
+                     $FTDB{"filter_fakedsmtp"}="$matchcount:$matchdate";
 
-                  my $append=append_message_to_folder($allmessageids[$i],
-   					\@attr, \$currmessage, 'mail-trash',
-   					$r_validfolders, $user);
-                  if ($append>=0) {
-                     $op='move';
-                     $matched=1;
+                     my $append=append_message_to_folder($allmessageids[$i],
+      					\@attr, \$currmessage, 'mail-trash',
+      					$r_validfolders, $user);
+                     if ($append>=0) {
+                        $op='move';
+                        $matched=1;
+                     }
                   }
                }
-            }
-         } # end of checking faked smtp
+            } # end of checking faked smtp
+
+         } # end of if enable_smartfilter
 
       } # end of if msg not verified
 
@@ -636,8 +684,7 @@ sub mailfilter {
    close ($folderhandle);
 
    $HDB{'METAINFO'}=metainfo($folderfile) if (!$ioerr);
-   dbmclose(%HDB);
-   filelock("$headerdb$config{'dbm_ext'}", LOCK_UN) if (!$config{'dbmopen_haslock'});
+   close_dbm(\%HDB, $headerdb);
 
    if (!$ioerr) {
       # remove repeated msgs with repeated count > $filter_repeatlimit
@@ -649,7 +696,11 @@ sub mailfilter {
          my $repeated;
          my ($trashfile, $trashdb)=get_folderfile_headerdb($user, 'mail-trash');
 
-         filelock($trashfile, LOCK_EX|LOCK_NB) or return -7;
+         if (! filelock($trashfile, LOCK_EX|LOCK_NB) ) {
+            close_dbm(\%FTDB, "$folderdir/.filter.book");
+            filelock($folderfile, LOCK_UN);
+            return -7;
+         }
          $repeated=operate_message_with_ids('move', \@repeatedids, $folderfile, $headerdb,
    							$trashfile, $trashdb);
          filelock($trashfile, LOCK_UN);
@@ -662,9 +713,7 @@ sub mailfilter {
       }
    }
 
-   dbmclose(%FTDB);
-   filelock("$folderdir/.filter.book$config{'dbm_ext'}", LOCK_UN) if (!$config{'dbmopen_haslock'});
-
+   close_dbm(\%FTDB, "$folderdir/.filter.book");
    filelock($folderfile, LOCK_UN);
 
    if (!$ioerr) {
@@ -693,10 +742,6 @@ sub append_message_to_folder {
    my %HDB2;
    my ($dstfile, $dstdb)=get_folderfile_headerdb($user, $destination);
    my $ioerr=0;
-
-   ($dstfile =~ /^(.+)$/) && ($dstfile = $1);  # untaint $dstfile
-   ($dstdb =~ /^(.+)$/) && ($dstdb = $1);  # untaint $dstdb
-
    if (${$r_currmessage} !~ /^From /) { # msg format error
       return -1;
    }
@@ -716,22 +761,18 @@ sub append_message_to_folder {
       return -4;
    }
 
-   if (!$config{'dbmopen_haslock'}) {
-      filelock("$dstdb$config{'dbm_ext'}", LOCK_EX) or return -5;
-   }
-   dbmopen (%HDB2, "$dstdb$config{'dbmopen_ext'}", 0600);
+   open_dbm(\%HDB2, $dstdb, LOCK_EX) or return -5;
+
    if (!defined($HDB2{$messageid}) ) {	# append only if not found in dstfile
       if (! open(DEST, ">>$dstfile")) {
-         dbmclose(%HDB2);
-         filelock("$dstdb$config{'dbm_ext'}", LOCK_UN) if (!$config{'dbmopen_haslock'});
-         filelock($dstfile, LOCK_UN);
+         close_dbm(\%HDB2, $dstdb);
          return -6;
       }
       seek(DEST, 0, 2);	# seek end explicitly to cover tell() bug in perl 5.8
       my @attr2=@{$r_attr};
       $attr2[$_OFFSET]=tell(DEST);
       $attr2[$_SIZE]=length(${$r_currmessage});
-      print DEST ${$r_currmessage} || $ioerr++;
+      print DEST ${$r_currmessage} or $ioerr++;
       close (DEST);
 
       if (!$ioerr) {
@@ -742,9 +783,8 @@ sub append_message_to_folder {
          $HDB2{'METAINFO'}=metainfo($dstfile);
       }
    }
-   dbmclose(%HDB2);
+   close_dbm(\%HDB2, $dstdb);
 
-   filelock("$dstdb$config{'dbm_ext'}", LOCK_UN) if (!$config{'dbmopen_haslock'});
    filelock($dstfile, LOCK_UN);
    return 0;
 }
