@@ -142,8 +142,7 @@ sub _encode_B {
 
 
 # this is used to decode fileblock generated by uuencode program
-sub uudecode ($)
-{
+sub uudecode ($) {
     local($^W) = 0; # unpack("u",...) gives bogus warning in 5.00[123]
 
     my $str=shift;
