@@ -1,12 +1,12 @@
-#!/usr/bin/perl -T
+#!/usr/bin/suidperl -T
 #
 # openwebmail-abook.pl - address book program
 #
 
 use vars qw($SCRIPT_DIR);
-if ( $0 =~ m!^(.*?)/[\w\d\-\.]+\.pl! ) { $SCRIPT_DIR=$1; }
-if (!$SCRIPT_DIR && open(F, '/var/www/conf/openwebmail/openwebmail_path.conf')) {
-   $_=<F>; close(F); if ( $_=~/^([^\s]*)/) { $SCRIPT_DIR=$1; }
+if ( $0 =~ m!^(\S*)/[\w\d\-\.]+\.pl! ) { $SCRIPT_DIR=$1; }
+if (!$SCRIPT_DIR && open(F, '/etc/openwebmail_path.conf')) {
+   $_=<F>; close(F); if ( $_=~/^(\S*)/) { $SCRIPT_DIR=$1; }
 }
 if (!$SCRIPT_DIR) { print "Content-type: text/html\n\nSCRIPT_DIR not set in /etc/openwebmail_path.conf !\n"; exit 0; }
 push (@INC, $SCRIPT_DIR);
@@ -46,6 +46,10 @@ $SIG{TERM}=\&openwebmail_exit;	# for user stop
 
 userenv_init();
 
+if (!$config{'enable_webmail'}) {
+   openwebmailerror(__FILE__, __LINE__, "$lang_text{'webmail'} $lang_err{'access_denied'}");
+}
+
 $page = param("page") || 1;
 $sort = param("sort") || $prefs{'sort'} || 'date';
 $messageid=param("message_id") || '';
@@ -793,8 +797,8 @@ sub editaddresses {
       }
 
       my ($k, $a, $n)=($key, $addresses{$key}, $notes{$key});
-      $k=~s/\\/\\\\/; $k=~s/'/\\'/; 
-      $a=~s/\\/\\\\/; $a=~s/'/\\'/; 
+      $k=~s/\\/\\\\/; $k=~s/'/\\'/;
+      $a=~s/\\/\\\\/; $a=~s/'/\\'/;
       $n=~s/\\/\\\\/; $n=~s/'/\\'/; # escape \ and ' for javascript
       $temphtml .= qq|<tr>|.
                    qq|<td bgcolor=$bgcolor><a $accesskeystr href="Javascript:Update('$k','$a','$n')">$namestr</a></td>|.
@@ -863,8 +867,8 @@ sub editaddresses {
       $emailstr=substr($emailstr, 0, 35)."..." if (length($emailstr)>40);
 
       my ($k, $a, $n)=($key, $globaladdresses{$key}, $globalnotes{$key});
-      $k=~s/\\/\\\\/; $k=~s/'/\\'/; 
-      $a=~s/\\/\\\\/; $a=~s/'/\\'/; 
+      $k=~s/\\/\\\\/; $k=~s/'/\\'/;
+      $a=~s/\\/\\\\/; $a=~s/'/\\'/;
       $n=~s/\\/\\\\/; $n=~s/'/\\'/; # escape \ and ' for javascript
       $temphtml .= qq|<tr>|.
                    qq|<td bgcolor=$bgcolor><a href="Javascript:Update('$k','$a','$n')">$namestr</a></td>|.
@@ -888,15 +892,13 @@ sub editaddresses {
 ################### MODADDRESS ##############################
 sub modaddress {
    my $mode = shift;
-   my ($realname, $address, $usernote);
-   $realname = param("realname") || '';
-   $address = param("email") || '';
-   $usernote = param("note") || '';
-   $realname =~ s/^\s*//; # strip beginning and trailing spaces from hash key
-   $address =~ s/[#&=\?]//g;
+   my $realname = param("realname") || '';
+   my $address = param("email") || '';
+   my $usernote = param("note") || '';
+   $realname =~ s/^\s*//; $realname =~ s/^\s*//;
+   $address =~ s/[\<\>\[\]\\:\`\"\s]//g;
    $address =~ s/^\s*mailto:\s*//;
-   $usernote =~ s/^\s*//; # strip beginning and trailing spaces
-   $usernote =~ s/\s*$//;
+   $usernote =~ s/^\s*//; $usernote =~ s/\s*$//;
 
    if (($realname && $address) || (($mode eq 'delete') && $realname) ) {
       my %addresses;
