@@ -1,12 +1,12 @@
-#!/usr/bin/perl -T
+#!/usr/bin/suidperl -T
 #
 # openwebmail-folder.pl - mail folder management program
 #
 
 use vars qw($SCRIPT_DIR);
-if ( $0 =~ m!^(.*?)/[\w\d\-\.]+\.pl! ) { $SCRIPT_DIR=$1; }
-if (!$SCRIPT_DIR && open(F, '/var/www/conf/openwebmail/openwebmail_path.conf')) {
-   $_=<F>; close(F); if ( $_=~/^([^\s]*)/) { $SCRIPT_DIR=$1; }
+if ( $0 =~ m!^(\S*)/[\w\d\-\.]+\.pl! ) { $SCRIPT_DIR=$1; }
+if (!$SCRIPT_DIR && open(F, '/etc/openwebmail_path.conf')) {
+   $_=<F>; close(F); if ( $_=~/^(\S*)/) { $SCRIPT_DIR=$1; }
 }
 if (!$SCRIPT_DIR) { print "Content-type: text/html\n\nSCRIPT_DIR not set in /etc/openwebmail_path.conf !\n"; exit 0; }
 push (@INC, $SCRIPT_DIR);
@@ -50,6 +50,10 @@ $SIG{TERM}=\&openwebmail_exit;	# for user stop
 
 userenv_init();
 
+if (!$config{'enable_webmail'}) {
+   openwebmailerror(__FILE__, __LINE__, "$lang_text{'webmail'} $lang_err{'access_denied'}");
+}
+
 $page = param("page") || 1;
 $sort = param("sort") || $prefs{'sort'} || 'date';
 
@@ -151,7 +155,7 @@ sub editfolders {
    my $form_i=0;
    $temphtml='';
    foreach $currfolder (@userfolders) {
-      $temphtml .= _folderline($currfolder, $form_i, $bgcolor, 
+      $temphtml .= _folderline($currfolder, $form_i, $bgcolor,
                                \$total_newmessages, \$total_allmessages, \$total_foldersize);
       if ($bgcolor eq $style{"tablerow_dark"}) {
          $bgcolor = $style{"tablerow_light"};
@@ -184,7 +188,7 @@ sub editfolders {
          $percent=int($quotausage*1000/$quotalimit)/10;
          $usagestr.=" ($percent%) &nbsp;";
          $usagestr.="$lang_text{'quotalimit'}: ".lenstr($quotalimit*1024,1);
-      }         
+      }
       if ($percent>=90) {
          $usagestr="<B><font color='#cc0000'>$usagestr</font></B>";
       } else {
@@ -217,11 +221,8 @@ sub _folderline {
    my ($folderfile,$headerdb)=get_folderfile_headerdb($user, $currfolder);
 
    if ( -f "$headerdb$config{'dbm_ext'}" && !-z "$headerdb$config{'dbm_ext'}" ) {
-      if (!$config{'dbmopen_haslock'}) {
-         filelock("$headerdb$config{'dbm_ext'}", LOCK_SH) or
+      open_dbm(\%HDB, $headerdb, LOCK_SH) or
             openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_locksh'} $headerdb$config{'dbm_ext'}");
-      }
-      dbmopen (%HDB, "$headerdb$config{'dbmopen_ext'}", undef);
       if ( defined($HDB{'ALLMESSAGES'}) ) {
          $allmessages=$HDB{'ALLMESSAGES'};
          ${$r_total_allmessages}+=$allmessages;
@@ -234,8 +235,7 @@ sub _folderline {
       } else {
          $newmessages='&nbsp;';
       }
-      dbmclose(%HDB);
-      filelock("$headerdb$config{'dbm_ext'}", LOCK_UN) if (!$config{'dbmopen_haslock'});
+      close_dbm(\%HDB, $headerdb);
    } else {
       $allmessages='&nbsp;';
       $newmessages='&nbsp;';
@@ -343,7 +343,7 @@ sub refreshfolders {
    writehistory("folder - refresh, $errcount errors");
 
    # get uptodate quota/folder usage info
-   getfolders(\@validfolders, \$folderusage);   
+   getfolders(\@validfolders, \$folderusage);
    if ($config{'quota_module'} ne 'none') {
       $quotausage=(quota_get_usage_limit(\%config, $user, $homedir, 1))[2];
    }
@@ -366,11 +366,8 @@ sub markreadfolder {
    }
 
    my (%HDB, %offset, %status);
-   if (!$config{'dbmopen_haslock'}) {
-      filelock("$headerdb$config{'dbm_ext'}", LOCK_SH) or
+   open_dbm(\%HDB, $headerdb, LOCK_SH) or
          openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_locksh'} $headerdb$config{'dbm_ext'}");
-   }
-   dbmopen (%HDB, "$headerdb$config{'dbmopen_ext'}", undef);
    foreach my $messageid (keys %HDB) {
       next if ( $messageid eq 'METAINFO'
              || $messageid eq 'NEWMESSAGES'
@@ -383,8 +380,7 @@ sub markreadfolder {
          $status{$messageid}=$attr[$_STATUS];
       }
    }
-   dbmclose(%HDB);
-   filelock("$headerdb$config{'dbm_ext'}", LOCK_UN) if (!$config{'dbmopen_haslock'});
+   close_dbm(\%HDB, $headerdb);
 
    my @markids;
    my $tmpfile="/tmp/markread_tmp_$$";
@@ -398,7 +394,7 @@ sub markreadfolder {
 
    if (update_headerdb($tmpdb, $tmpfile)<0) {
       filelock($tmpfile, LOCK_UN);
-      unlink("$tmpdb$config{'dbm_ext'}", $tmpfile);
+      unlink("$tmpdb$config{'dbm_ext'}", "$tmpdb.dir", "$tmpdb.pag", $tmpfile);
       openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_updatedb'} $tmpdb$config{'dbm_ext'}");
    }
 
@@ -425,7 +421,7 @@ sub markreadfolder {
 
    filelock("$tmpfile", LOCK_UN);
    filelock($folderfile, LOCK_UN);
-   unlink("$tmpdb$config{'dbm_ext'}", $tmpfile);
+   unlink("$tmpdb$config{'dbm_ext'}", "$tmpdb.dir", "$tmpdb.pag", $tmpfile);
 
    writelog("markread folder - $foldertomark");
    writehistory("markread folder - $foldertomark");
@@ -443,21 +439,17 @@ sub reindexfolder {
    my ($folderfile, $headerdb)=get_folderfile_headerdb($user, $foldertoindex);
 
    if ($recreate) {
-      my $filename="$headerdb$config{'dbm_ext'}";
-      ($filename =~ /^(.+)$/) && ($filename = $1);  # untaint
+      my $filename=untaint("$headerdb$config{'dbm_ext'}");
       unlink($filename);
    }
 
    if ( -f "$headerdb$config{'dbm_ext'}" ) {
       my %HDB;
-      if (!$config{'dbmopen_haslock'}) {
-         filelock("$headerdb$config{'dbm_ext'}", LOCK_SH) or
+      open_dbm(\%HDB, $headerdb, LOCK_SH) or
             openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_locksh'} $headerdb$config{'dbm_ext'}");
-      }
-      dbmopen (%HDB, "$headerdb$config{'dbmopen_ext'}", 0600);
+
       $HDB{'METAINFO'}={'RENEW'};
-      dbmclose(%HDB);
-      filelock("$headerdb$config{'dbm_ext'}", LOCK_UN) if (!$config{'dbmopen_haslock'});
+      close_dbm(\%HDB, $headerdb);
    }
    if (update_headerdb($headerdb, $folderfile)<0) {
       filelock($folderfile, LOCK_UN);
@@ -509,8 +501,9 @@ sub addfolder {
 
    # create empty index dbm with mode 0600
    my %HDB;
-   dbmopen(%HDB, "$headerdb$config{'dbmopen_ext'}", 0600);
-   dbmclose(%HDB);
+   open_dbm(\%HDB, $headerdb, LOCK_EX) or
+      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_lock'} $headerdb$config{'dbm_ext'}");
+   close_dbm(\%HDB, $headerdb);
 
    writelog("create folder - $foldertoadd");
    writehistory("create folder - $foldertoadd");
@@ -667,7 +660,7 @@ sub downloadfolder {
    }
    print qq|\n|;
 
-   ($cmd =~ /^(.+)$/) && ($cmd = $1);		# untaint ...
+   $cmd=untaint($cmd);
    open (T, $cmd);
    while ( read(T, $buff,32768) ) {
      print $buff;
