@@ -1,12 +1,12 @@
-#!/usr/bin/perl -T
+#!/usr/bin/suidperl -T
 #
 # openwebmail-viewatt.pl - attachment reading program
 #
 
 use vars qw($SCRIPT_DIR);
-if ( $0 =~ m!^(.*?)/[\w\d\-\.]+\.pl! ) { $SCRIPT_DIR=$1; }
-if (!$SCRIPT_DIR && open(F, '/var/www/conf/openwebmail/openwebmail_path.conf')) {
-   $_=<F>; close(F); if ( $_=~/^([^\s]*)/) { $SCRIPT_DIR=$1; }
+if ( $0 =~ m!^(\S*)/[\w\d\-\.]+\.pl! ) { $SCRIPT_DIR=$1; }
+if (!$SCRIPT_DIR && open(F, '/etc/openwebmail_path.conf')) {
+   $_=<F>; close(F); if ( $_=~/^(\S*)/) { $SCRIPT_DIR=$1; }
 }
 if (!$SCRIPT_DIR) { print "Content-type: text/html\n\nSCRIPT_DIR not set in /etc/openwebmail_path.conf !\n"; exit 0; }
 push (@INC, $SCRIPT_DIR);
@@ -53,6 +53,10 @@ $SIG{TERM}=\&openwebmail_exit;	# for user stop
 
 userenv_init();
 
+if (!$config{'enable_webmail'}) {
+   openwebmailerror(__FILE__, __LINE__, "$lang_text{'webmail'} $lang_err{'access_denied'}");
+}
+
 $page = param("page") || 1;
 $sort = param("sort") || $prefs{'sort'} || 'date';
 $keyword = param("keyword") || '';
@@ -412,8 +416,7 @@ sub savefile2webdisk {
       }
    }
 
-   my $webdiskrootdir=$homedir.absolute_vpath("/", $config{'webdisk_rootpath'});
-   ($webdiskrootdir =~ m!^(.+)/?$!) && ($webdiskrootdir = $1);  # untaint ...
+   my $webdiskrootdir=untaint($homedir.absolute_vpath("/", $config{'webdisk_rootpath'}));
    my $vpath=absolute_vpath('/', $webdisksel);
    my $err=verify_vpath($webdiskrootdir, $vpath);
    openwebmailerror(__FILE__, __LINE__, $err) if ($err);
@@ -423,7 +426,7 @@ sub savefile2webdisk {
       $err=verify_vpath($webdiskrootdir, $vpath);
       openwebmailerror(__FILE__, __LINE__, $err) if ($err);
    }
-   ($vpath =~ /^(.+)$/) && ($vpath = $1);  # untaint ...
+   $vpath=untaint($vpath);
 
    if (!open(F, ">$webdiskrootdir/$vpath") ) {
       autoclosewindow($lang_text{'savefile'}, "$lang_text{'savefile'} $lang_text{'failed'} ($vpath: $!)");
