@@ -1,8 +1,8 @@
 #
 # pop3mail.pl - pop3 mail retrieval routines
 #
-# 2003/05/25 tung@turtle.ee.ncku.edu.tw
-# 2002/03/19 eddie@turtle.ee.ncku.edu.tw
+# 2003/05/25 tung.AT.turtle.ee.ncku.edu.tw
+# 2002/03/19 eddie.AT.turtle.ee.ncku.edu.tw
 #
 
 use strict;
@@ -14,7 +14,7 @@ use vars qw(%config);
 
 # return < 0 means error
 # -1 uidldb lock error
-# -2 uidldb open error
+# -2 uidldb open error (unused)
 # -3 spool write error
 # -11 connect error
 # -12 server not ready
@@ -27,13 +27,10 @@ sub retrpop3mail {
    my ($pop3host, $pop3port, $pop3user, $pop3passwd, $pop3del, $uidldb, $spoolfile)=@_;
    my $remote_sock;
 
-   # untaint for connection creation
-   ($pop3host =~ /^(.+)$/) && ($pop3host = $1);
-   ($pop3port =~ /^(.+)$/) && ($pop3port = $1);
-   # untaint for file creation
-   ($spoolfile =~ /^(.+)$/) && ($spoolfile = $1);
-   # untaint for uidldb creation
-   ($uidldb =~ /^(.+)$/) && ($uidldb = $1);		# untaint ...
+   $pop3host=untaint($pop3host);	# untaint for connection creation
+   $pop3port=untaint($pop3port);
+   $spoolfile=untaint($spoolfile);	# untaint for file creation
+   $uidldb=untaint($uidldb);		# untaint for uidldb creation
 
    eval {
       local $SIG{ALRM} = sub { die "alarm\n" }; # NB: \n required
@@ -111,16 +108,9 @@ sub retrpop3mail {
       } else {
          $uidl_field=1;	# some broken pop3d return uidl without leading +
       }
-      if (!$config{'dbmopen_haslock'}) {
-         if (!filelock("$uidldb$config{'dbm_ext'}", LOCK_EX|LOCK_NB)) {
-            close($remote_sock);
-            return -1;
-         }
-      }
-      if (!dbmopen(%UIDLDB, "$uidldb$config{'dbmopen_ext'}", 0600)) {
-         filelock("$uidldb$config{'dbm_ext'}", LOCK_UN) if (!$config{'dbmopen_haslock'});
+      if (!open_dbm(\%UIDLDB, $uidldb, LOCK_EX|LOCK_NB)) {
          close($remote_sock);
-         return -2;
+         return -1;
       }
    }
 
@@ -136,7 +126,7 @@ sub retrpop3mail {
             $uidldb{$uidl}=1; next;
          }
       }
-          
+
       print $remote_sock "retr ".$i."\r\n";
       while (<$remote_sock>) {	# use loop to filter out verbose output
          if ( /^\+/ ) {
@@ -144,8 +134,7 @@ sub retrpop3mail {
          } elsif (/^\-/) {
             if ($uidl_support) {
                @UIDLDB{keys %uidldb}=values %uidldb if ($retr_total>0);
-               dbmclose(%UIDLDB);
-               filelock("$uidldb$config{'dbm_ext'}", LOCK_UN) if (!$config{'dbmopen_haslock'});
+               close_dbm(\%UIDLDB, $uidldb);
             }
             close($remote_sock);
             return -17;
@@ -210,14 +199,13 @@ sub retrpop3mail {
             print IN "\n";		# mark mail end
             close(IN);
             $append=1;
-         } 
+         }
          filelock($spoolfile, LOCK_UN);
-      } 
+      }
       if (!$append) {
          if ($uidl_support) {
             @UIDLDB{keys %uidldb}=values %uidldb if ($retr_total>0);
-            dbmclose(%UIDLDB);
-            filelock("$uidldb$config{'dbm_ext'}", LOCK_UN) if (!$config{'dbmopen_haslock'});
+            close_dbm(\%UIDLDB, $uidldb);
          }
          close($remote_sock);
          return -3;
@@ -238,8 +226,7 @@ sub retrpop3mail {
 
    if ($uidl_support) {
       %UIDLDB=%uidldb if ($retr_total>0);
-      dbmclose(%UIDLDB);
-      filelock("$uidldb$config{'dbm_ext'}", LOCK_UN) if (!$config{'dbmopen_haslock'});
+      close_dbm(\%UIDLDB, $uidldb);
    }
 
    # return number of fetched mail
